<%
''''''''''''''''''''''''''''''''''''''''''''''''''''
' MycoDB - online database of mycorrhizal experiments 
' Copyright (C) 2007  Lawrence Walters
'
' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.
' 
' $RCSfile: database.inc.asp,v $
' $Date: 2006/01/20 20:40:19 $
' $Revision: 1.13 $
' Purpose:	database functions
' Author:		Lawrence Walters
'
'	modified:
'	2005-08-03	LLW	no longer uses recordset.update and .addnew in DbInput - converted to sql statements
'	2005-10-24	LLW allow apostrophes in insert statements menubar
'	2006-01-18	LLW	dbBuildSelect no has built in yes/no/unknown options, and no longer supports null
''''''''''''''''''''''''''''''''''''''''''''''''''''
dim gConn ' global connection variable
dim isDevMode ' global development mode indicator
dim showDebugInfo ' global flag for showing/hiding debug info

const DB_FIELD = "db_"

'<!--METADATA
'	TYPE="TypeLib"
'	NAME="Microsoft ActiveX Data Objects 2.5 Library"
'	UUID="{00000205-0000-0010-8000-00AA006D2EA4}"
'	VERSION="2.5"
'-->

' CursorTypeEnum
Const adOpenForwardOnly = 0
Const adOpenKeyset = 1
Const adOpenDynamic = 2
Const adOpenStatic = 3
' LockTypeEnum
Const adLockReadOnly = 1
Const adLockPessimistic = 2
Const adLockOptimistic = 3
' CommandTypeEnum
Const adCmdText = 1
Const adCmdTable = 2

' execute options enum
const adAsyncFetchNonBlocking = 64

Const adStateClosed = &H00000000 
Const adStateOpen = &H00000001 
Const adStateConnecting = &H00000002 
Const adStateExecuting = &H00000004 

Const adTypeBinary = 1
Const adLongVarChar = 201
Const adLongVarBinary = 205
Const adSaveCreateOverWrite = 2


'******************************************************************
'Function:			dbConnect
'Description:	connect to the database
'Parameters:		none
'Return:				none
'Author:				Lawrence Walters
'*******************************************************************/
sub dbConnect()
	dim MdbFilePath
	dim dsn
	dim sql, rs
	
	MdbFilePath = Server.MapPath("fpdb/mycodb.mdb")
	dsn = "Driver={Microsoft Access Driver (*.mdb)}; DBQ=" & MdbFilePath & ";"

	set gConn = nothing
	set gConn = Server.CreateObject("ADODB.Connection")
	gConn.Open dsn, "", ""
	
	'check if we're in development mode
	isDevMode = false
	set rs = Server.CreateObject("ADODB.Recordset")
	sql = "select [value] from config where [name] = 'DevelopmentMode'"
	rs.open sql, gConn, adOpenForwardOnly
	if not rs.eof then
		if rs("value") = 1 then
			isDevMode = true
		end if
		rs.close
	end if
	
	'check if we should show debug info mode
	showDebugInfo = false
	set rs = Server.CreateObject("ADODB.Recordset")
	sql = "select [value] from config where [name] = 'ShowDebugInfo'"
	rs.open sql, gConn, adOpenForwardOnly
	if not rs.eof then
		if rs("value") = 1 then
			showDebugInfo = true
		end if
		rs.close
	end if
	
	set rs = nothing
end sub

'******************************************************************
'Function:			dbClose
'Description:		close database connection
'Parameters:		none
'Return:				none
'Author:				Lawrence Walters
'*******************************************************************/
sub dbClose()
	gConn.close
end sub

'******************************************************************
'Function:			dbInput
'Description:		insert a new record or update an existing one
'Parameters:		table - name of table to update
'								errors - dictionary for error strings
'Return:				id of new/updated record
'Author:				Lawrence Walters
'*******************************************************************/
function dbInput(table, byref errors)
	dim cnt, errormessage
	dim fieldnames(), fieldvalues()
	dim rs, insert, sql

	if len(Request.Form(DB_FIELD & "id")) > 0 then
		if Request.Form(DB_FIELD & "id") = "new" then
			insert = true
		else
			insert = false
		end if
	else
		errors.add errors.count + 1, "No id specified - record not updated."
		exit function
	end if
	
	'table name defined by "_dbTable" field
	if len(table) = 0 then
		errors.add errors.count + 1, "No table specified - record not updated."
		exit function
	end if
	
	'fields defined by dbFieldName fields
	if Request.Form.Count > 0 then
		cnt = 0
		
		for each objItem in Request.Form
			if left(objItem, len(DB_FIELD)) = DB_FIELD then
				' skip id field when this is a new record
				if not (objItem = DB_FIELD & "id") then
					redim preserve fieldnames(cnt)
					redim preserve fieldvalues(cnt)
					fieldnames(cnt) = mid(objItem, len(DB_FIELD) + 1)
					if len(request.form(objItem)) > 0 and request.form(objItem) <> "null" then
						fieldvalues(cnt) = Request.Form(objItem)
						fieldvalues(cnt) = replace(fieldvalues(cnt), "'", "''")
					else
						fieldvalues(cnt) = null
					end if
					
					
					cnt = cnt + 1
				end if
			end if
		next
		
		' for debugging
		response.write "Table: " & table &"<BR><table border=1><Td>Field</td><td>Value</td>"
		for cnt = lbound(fieldnames) to ubound(fieldnames)
			response.write "<tr><td>" & fieldnames(cnt) & "</td><td>" & fieldvalues(cnt) & "</td></tr>"
		next
		response.write "</table>"
		'response.end
		'exit function
		
		'Set rs = Server.CreateObject("ADODB.RecordSet")
		
		' add a new record
		if insert then
			' build sql statement with field names
			sql = "insert into [" & table & "] ("
			for cnt = lbound(fieldnames) to ubound(fieldnames)
				sql = sql & "[" & fieldnames(cnt) & "], "
			next
			
			' trim trailing comma
			sql = mid(sql, 1, len(sql) - 2)
			
			' add values to sql statement
			sql = sql & ") values("
			for cnt = lbound(fieldvalues) to ubound(fieldvalues)
				if IsNull(fieldvalues(cnt)) then
					sql = sql & "null"
				elseif isnumeric(fieldvalues(cnt)) then
					sql = sql & fieldvalues(cnt)
				else
					sql = sql & "'" & fieldvalues(cnt) & "'"
				end if
				sql = sql & ", "
			next
			
			' trim trailing comma
			sql = mid(sql, 1, len(sql) - 2)
			sql = sql & ")"

			' debugging
			'response.write sql & "<br>"			
			'response.end
			gConn.execute sql, , 129
			
			' get the identity of recently inserted record
			set rs = Server.CreateObject("ADODB.Recordset")
			sql = "select @@identity as id"
			rs.Open sql, gConn
			If Not rs.EOF Then
					dbInput = rs("id")
					rs.Close
			End If			
			set rs = nothing
			
		' update an existing record
		else
			sql = "update [" & table & "] set "
			for cnt = lbound(fieldvalues) to ubound(fieldvalues)
				sql = sql & fieldnames(cnt) & " = "
				if IsNull(fieldvalues(cnt)) then
					sql = sql & "null"
				elseif IsNumeric(fieldvalues(cnt)) then
					sql = sql & fieldvalues(cnt)
				else
					sql = sql & "'" & fieldvalues(cnt) & "'"
				end if
				sql = sql & ", "
			next
			sql = mid(sql, 1, len(sql) - 2)
			sql = sql & " where id = " & Request.Form(DB_FIELD & "id")
			
			'DEBUGGING
			'response.write sql & "<br>"
			'RESPONSE.END
			gConn.execute sql, , 129
			
			dbInput = Request.Form(DB_FIELD & "id")
		end if
	end if
end function

'******************************************************************
'Function:			dbBuildSelect
'Description:		create a select dropdown
'Parameters:		name - name of object to be created 
'								sql statement that returns 2 columns: value and display text
'								matchValue - value of item that is selected
'								includeNull - true/false - add a row at the beggining that is null
'								isYesNo - true/false - is this a yes/no/unknown dropdown box? If true, the sql statement will be ignored
'Return:				html
'Author:				Lawrence Walters
'*******************************************************************/
function dbBuildSelect(name, sql, matchValue, includeNull, isYesNo)
	dim retval, rs
	
	'todo: eliminate this parameter if we decide to really not use this function
	'includeNull = false
	
	' start of dropdown
	retval = "<select name=""" & DB_FIELD & name & """>" & vbcrlf
	
	if isYesNo then
		sql = "select -1, 'Unknown' from papers union select 0, 'No' from papers union select 1, 'Yes' from papers"
	end if
	
	' include a blank value?
	if includeNull then retval = retval & "<option value=""""> </option>" & vbcrlf

	' get all options	
	set rs = Server.CreateObject("ADODB.Recordset")
	rs.open sql, gConn, adOpenForwardOnly
	do while not rs.EOF
		retval = retval & "<option "
		
		' mark as selected
		if not isnull(rs(0)) and len(matchValue) > 0 then
		
			' if its a number, do a numeric comparison
			if IsNumeric(rs(0)) then
				if CLng(rs(0)) = CLng(matchValue) then
					retval = retval & "selected "
				end if
				
			' not a number
			else
				if rs(0) = matchValue then
					retval = retval & "selected "
				end if
			end if
		end if
		
		retval = retval & "value=""" & rs(0) & """>" & rs(1) & "</option>" & vbcrlf
		rs.MoveNext
	loop
	retval = retval & "</select>" & vbcrlf
	dbBuildSelect = retval
end function

'******************************************************************
'Function:			dbRead
'Description:		create a select dropdown
'Parameters:		table - database table name 
'								recordId - id in database table
'Return:				dictionary object with fielname, data as key, value pairs
'Author:				Lawrence Walters
'*******************************************************************/
function dbRead(table, recordId)
	dim dict, rs, sql, field
	
	set dict = Server.CreateObject("Scripting.Dictionary")
	set rs = Server.CreateObject("ADODB.Recordset")
	sql = "select * from [" & table & "] where [id] = " & recordId
	rs.open sql, gConn, adOpenForwardOnly
	
	if not rs.eof then
		for each field in rs.fields
			' debugging:
			'response.write "[" & field.Name & "] = '" & field.Value & "'<br>"
			dict.add field.Name, field.Value
		next
	end if
	
	set dbRead = dict
end function

'******************************************************************
'Function:			FormRead
'Description:		read all form inputs, and for each dbXxxx variable, add it to a data dictionary
'Parameters:		none
'Return:				data dictionary
'Author:				Lawrence Walters
'******************************************************************/
function FormRead()
	dim objItem, dict
	set dict = server.createobject("Scripting.Dictionary")
	
	' add every item that starts with DB_FIELD
	for each objItem in Request.Form
		if left(objItem, len(DB_FIELD)) = DB_FIELD then
			dict.add mid(objItem, len(DB_FIELD) + 1), Request.Form(objItem)
		end if
	next
	
	set FormRead = dict
end function

'******************************************************************
'Function:			dbDelete
'Description:		delete a record
'Parameters:		table - name of table to update
'								id of record to delete
'								errors - dictionary for error strings
'Return:				none
'Author:				Lawrence Walters
'*******************************************************************/
sub dbDelete(table, id, byref errors)
	dim cnt, errormessage
	dim sql

	sql = "delete from [" & table & "] where id = " & id
	gConn.Execute sql
	
end sub

%>
