<%@ Language="VBScript"%>
<%
option explicit
''''''''''''''''''''''''''''''''''''''''''''''''''''
' MycoDB - online database of mycorrhizal experiments 
' Copyright (C) 2007  Lawrence Walters
'
' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.
' 
' $RCSfile: searchExport.asp,v $
' $Date: 2005/12/03 00:06:05 $
' $Revision: 1.2 $
' Purpose:	export some data, duh... will eventually allow user to specify some criteria
' Author:		Lawrence Walters
'
''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' include files/database setup
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
PageTitle = "Search - Export Data"
%>
<!-- #INCLUDE FILE="include/init.inc.asp" -->
<%
RequireLogin()


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' constants
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

const OUTPUT_ROW_LIMIT = 0 'limit the number of rows. Set to 0 for no limit
const OUTPUT_CSV_FILE = true 'set to true to generate file. false for debugging
const OUTPUT_DATA = true 'set to true to actually display data


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' declare classes
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

' the result class simply contains the 6 possible result values that we track
class Result
	public mean
	public stdError
	public stdDev
	public confInter95
	public lsd
	public numReplicates
	
	Private Sub Class_Initialize()
		mean = null
		stdError = null
		stdDev = null
		confInter95 = null
		lsd = null
		numReplicates = null
	end sub	
end class

' this class contains all the information that will be displayed in a single row of the output
class ResultRow
	public results 'dictionary containing Result objects for non control results with response id as key
	public ctlResults 'dictionary containing Result objects for control results with response id as key
	public experimentDetails ' dictionary containing all the experiment parameters, keyed to parameter name
	
	public calcColumns ' dictionary containing extra calculated columns
	
	' constructor - define the dictionary objects
	Private Sub Class_Initialize()
		set results = Server.CreateObject("Scripting.Dictionary")
		set ctlResults = Server.CreateObject("Scripting.Dictionary")
		set experimentDetails = Server.CreateObject("Scripting.Dictionary")
		set calcColumns = Server.CreateObject("Scripting.Dictionary")
	End Sub
end class

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' declare variables
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

' Critical values for t (two-tailed).
' taken from http://davidmlane.com/hyperstat/t-table.html
' Use these for the calculation of confidence intervals. For example, use the 0.05 column for the 95% confidence interval.
dim T_TABLE(100)
T_TABLE(2) = 4.3027
T_TABLE(3) = 3.1824
T_TABLE(4) = 2.7765
T_TABLE(5) = 2.5706
T_TABLE(6) = 2.4469
T_TABLE(7) = 2.3646
T_TABLE(8) = 2.306
T_TABLE(9) = 2.2622
T_TABLE(10) = 2.2281
T_TABLE(11) = 2.201
T_TABLE(12) = 2.1788
T_TABLE(13) = 2.1604
T_TABLE(14) = 2.1448
T_TABLE(15) = 2.1315
T_TABLE(16) = 2.1199
T_TABLE(17) = 2.1098
T_TABLE(18) = 2.1009
T_TABLE(19) = 2.093
T_TABLE(20) = 2.086
T_TABLE(21) = 2.0796
T_TABLE(22) = 2.0739
T_TABLE(23) = 2.0687
T_TABLE(24) = 2.0639
T_TABLE(25) = 2.0595
T_TABLE(26) = 2.0555
T_TABLE(27) = 2.0518
T_TABLE(28) = 2.0484
T_TABLE(29) = 2.0452
T_TABLE(30) = 2.0423
T_TABLE(31) = 2.0395
T_TABLE(32) = 2.0369
T_TABLE(33) = 2.0345
T_TABLE(34) = 2.0322
T_TABLE(35) = 2.0301
T_TABLE(36) = 2.0281
T_TABLE(37) = 2.0262
T_TABLE(38) = 2.0244
T_TABLE(39) = 2.0227
T_TABLE(40) = 2.0211
T_TABLE(41) = 2.0195
T_TABLE(42) = 2.0181
T_TABLE(43) = 2.0167
T_TABLE(44) = 2.0154
T_TABLE(45) = 2.0141
T_TABLE(46) = 2.0129
T_TABLE(47) = 2.0117
T_TABLE(48) = 2.0106
T_TABLE(49) = 2.0096
T_TABLE(50) = 2.0086
T_TABLE(51) = 2.0076
T_TABLE(52) = 2.0066
T_TABLE(53) = 2.0057
T_TABLE(54) = 2.0049
T_TABLE(55) = 2.004
T_TABLE(56) = 2.0032
T_TABLE(57) = 2.0025
T_TABLE(58) = 2.0017
T_TABLE(59) = 2.001
T_TABLE(60) = 2.0003
T_TABLE(61) = 1.9996
T_TABLE(62) = 1.999
T_TABLE(63) = 1.9983
T_TABLE(64) = 1.9977
T_TABLE(65) = 1.9971
T_TABLE(66) = 1.9966
T_TABLE(67) = 1.996
T_TABLE(68) = 1.9955
T_TABLE(69) = 1.9949
T_TABLE(70) = 1.9944
T_TABLE(71) = 1.9939
T_TABLE(72) = 1.9935
T_TABLE(73) = 1.993
T_TABLE(74) = 1.9925
T_TABLE(75) = 1.9921
T_TABLE(76) = 1.9917
T_TABLE(77) = 1.9913
T_TABLE(78) = 1.9908
T_TABLE(79) = 1.9905
T_TABLE(80) = 1.9901
T_TABLE(81) = 1.9897
T_TABLE(82) = 1.9893
T_TABLE(83) = 1.989
T_TABLE(84) = 1.9886
T_TABLE(85) = 1.9883
T_TABLE(86) = 1.9879
T_TABLE(87) = 1.9876
T_TABLE(88) = 1.9873
T_TABLE(89) = 1.987
T_TABLE(90) = 1.9867
T_TABLE(91) = 1.9864
T_TABLE(92) = 1.9861
T_TABLE(93) = 1.9858
T_TABLE(94) = 1.9855
T_TABLE(95) = 1.9852
T_TABLE(96) = 1.985
T_TABLE(97) = 1.9847
T_TABLE(98) = 1.9845
T_TABLE(99) = 1.9842
T_TABLE(100) = 1.984


dim outputRowCount ' count the number of rows output

dim singleExperiment() ' array to contain multiple ResultRow objects that are within a single experiment
dim lastExperimentId ' track the last experiment id
	
'for calc'ing standard dev from LSD
dim expNumTreatmentLevels 'dictionary for number of treatment levels: key = treatment id, value = pipe separated string with levels.
dim expNumReplicates 'dictionary for tracking number of replicates for each experiment by treatment set

dim row 'single ResultRow of results

dim fileHeader ' header row with column names

dim tmpResult

dim field
dim key

dim resultValue
dim resultList
dim masterResultList

dim requestedTreatmentIds
dim treatmentList
dim masterTreatmentIds
dim masterTreatmentList
dim allLevels ' temp placeholder for all levels
dim levelString ' wrapped level text.
dim tmpTreatList 'temporary list of treatment levels for a treatment set
dim treatmentId 'treatment id

dim responseId
dim requestedResponseIds
dim responseList

dim sql
dim sqlNonCtlTreatments
dim sqlCtlResults
dim sqlNonCtlResults

dim rsExpTrtSets
dim rsResponses
dim rsTreatments
dim rsTreatmentLevels
dim rsCtlResults
dim rsNonCtlResults

dim calcStdDev 'did the user request standard deviation? we'll need to extra calculations if so

dim convertSoilNAndPToPPM 'did the user request to convert to ppm?
dim units ' units name for converting units
dim value 'placeholder for converting units

dim calculateNtoPRatios ' did the user request calculation of n to p ratios?
dim calcSumN ' calculate sum(n)? (control n and nont control n)
dim calcSumNRecip ' calculate 1 / sum(n) ?
dim calcLnResponseRatio ' calculate natural log of ( mean(inoculated) / mean(control) )

'for tracking control treatment set ids for random export
dim tmpString
dim treatmentSetId
dim treatmentSetRows
dim selectedRow ' the row chosen from the same control treatment set id
dim expCtlTrtSetIds ' dictionary for tracking control treatment set ids for random export

Dim PageTitle, FocusObject

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' initialize variables
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

' returned data will be comma separated values file
if OUTPUT_CSV_FILE then
	key = "MycoDbData"
	if request.form("Export") = "Random Subset" then
		key = key & "_random_"
	end if
	key = key & "_" & formatDate("%Y_%m_%d_%h%i", 0)
	
	Response.ContentType = "text/csv"
	Response.AddHeader "Content-Disposition", "attachment; filename=" & key  & ".csv"
else
	response.write "Date: " & formatDate("%Y_%m_%d_%h%i", 0) & "<br />"
end if

Randomize

' did the user request n to p ratio calculations?
if request.form("calcSumN") = "on" then
	calcSumN = true
else
	calcSumN = false
end if

' did the user request n to p ratio calculations?
if request.form("calcSumNRecip") = "on" then
	calcSumNRecip = true
else
	calcSumNRecip = false
end if

' did the user request n to p ratio calculations?
if request.form("calcLnResponseRatio") = "on" then
	calcLnResponseRatio = true
else
	calcLnResponseRatio = false
end if

' did the user request n to p ratio calculations?
if request.form("calculateNtoPRatios") = "on" then
	calculateNtoPRatios = true
else
	calculateNtoPRatios = false
end if

' did the user request unit conversion?
if request.form("convertSoilNAndPToPPM") = "on" then
	convertSoilNAndPToPPM = true
else
	convertSoilNAndPToPPM = false
end if

set expNumTreatmentLevels = Server.CreateObject("Scripting.Dictionary")
' always add "control" for inoculum treatment by default
expNumTreatmentLevels.add 4, "[control]"
set expNumReplicates = Server.CreateObject("Scripting.Dictionary")

lastExperimentId = -1

redim singleExperiment(0)

set rsExpTrtSets = server.createobject("ADODB.Recordset")
set rsResponses = server.createobject("ADODB.Recordset")
set rsTreatments = server.createobject("ADODB.Recordset")
set rsTreatmentLevels = server.createobject("ADODB.Recordset")
set rsCtlResults = server.createobject("ADODB.Recordset")
set rsNonCtlResults = server.createobject("ADODB.Recordset")

' make a list of all the result values
' this also holds the abbreviated column names for efficient output and usage in MetaWin
set masterResultList = Server.CreateObject("Scripting.Dictionary")
masterResultList.add "mean", "m"
masterResultList.add "stdError", "se"
masterResultList.add "stdDev", "sd"
masterResultList.add "confInter95", "ci"
masterResultList.add "lsd", "lsd"
masterResultList.add "numReplicates", "n"

' make an array of the values we track in results
set resultList = Server.CreateObject("Scripting.Dictionary")
for each key in request.form
	if left(key, len(DB_FIELD & "result_")) = DB_FIELD & "result_" then
		resultValue = mid(key, len(DB_FIELD & "result_") + 1)
		resultList.add resultValue, resultValue
	end if
next

' check if the user selected standard deviation.
if resultList.Exists("stdDev") then
	calcStdDev = true
else
	calcStdDev = false
end if

' get all treatments
' make a list of all the treatments
set masterTreatmentList = Server.CreateObject("Scripting.Dictionary")
set treatmentList = Server.CreateObject("Scripting.Dictionary")
sql = "select id, name from treatments order by displayOrder, name"
rsTreatments.open sql, gConn, adOpenForwardOnly

if not rsTreatments.eof then
	do while not rsTreatments.eof
	
		' add to master treatment list
		masterTreatmentList.add Trim(CStr(rsTreatments("id"))), Trim(CStr(rsTreatments("name")))
		if len(masterTreatmentIds) > 0 then
			masterTreatmentIds = masterTreatmentIds & ","
		end if	
		masterTreatmentIds = masterTreatmentIds & rsTreatments("id")	

		' check if these were selected by the user
		' but always include inoculum
		if len(Request.form(DB_FIELD & "treatment_" & rsTreatments("id"))) > 0 or rsTreatments("id") = 4 then
			treatmentList.add Trim(CStr(rsTreatments("id"))), Trim(CStr(rsTreatments("name")))
			if len(requestedTreatmentIds) > 0 then
				requestedTreatmentIds = requestedTreatmentIds & ","
			end if
			requestedTreatmentIds = requestedTreatmentIds & rsTreatments.fields.item("id").value
		end if
		rsTreatments.movenext
	loop
	
	rsTreatments.close
end if
set rsTreatments = nothing

'response.write "master treatment id's: " & masterTreatmentIds & "<Br />"

' get all responses
set responseList = Server.CreateObject("Scripting.Dictionary")
sql = "select id, name from responses order by CStr(id)"
rsResponses.open sql, gConn, adOpenForwardOnly

if not rsResponses.eof then
	do while not rsResponses.eof
		' check if these were selected by the user
		if len(Request.form(DB_FIELD & "response_" & rsResponses("id"))) > 0 then
			responseList.add Trim(CStr(rsResponses.fields.item("id").value)), Trim(CStr(rsResponses("name")))

			if len(requestedResponseIds) > 0 then
				requestedResponseIds = requestedResponseIds & ","
			end if
			requestedResponseIds = requestedResponseIds & rsResponses.fields.item("id").value

		end if

		rsResponses.movenext
	loop
	rsResponses.close
end if
set rsResponses = nothing

' get all the experiment treatment sets - each treatment set with its treatments and results corresponds to one row in the output data
sql = "select experimentId, name as experimentName, lastName1, lastName2, paperYear, journalName, paperTitle, userName, dateEntered, ecosystem, climate, country, closestCity, environment, mycorrhizaeType, year, month, soilNitrogen, soilNNutrientMeasurement, soilNNutrientUnit, soilNNutrientType, soilPhosphorous, soilPNutrientMeasurement, soilPNutrientUnit, soilPNutrientType, sterilizationMethod, hostPlantNative, plantLifeHistory, plantPhotoPath, plantGrowthHabit, plantFunctionalGroup, numWeeks, hostGenus, hostSpecies, hostPlantFamily, plantSource,  inoculumSpecies, growthMedium, timeCourseStudy, experimentalUnit, experimentalPotSize, inocFieldSoil, inocRoots, inocSpores, inocMycelia, inocPotCulture, fungiSource, nonMycoControl, fertN, fertP, fertK, micronutrients, nonCtlTrtSetId, ctlTrtSetId, sortKey from ExportTreatments where ctlTrtSetId > 0 order by ctlTrtSetId"

rsExpTrtSets.Open sql, gConn, adOpenForwardOnly
if not rsExpTrtSets.eof then

	' write all the field headers in the export query
	for each field in rsExpTrtSets.fields
		fileHeader = fileHeader & field.name & ","
	next
	
	' show treatments headers
	for each treatmentId in treatmentList
		' if this is inoculum, show it twice - once for the control level, once for non control
		if treatmentId = 4 then
			fileHeader = fileHeader & "c." & treatmentList.item(treatmentId) & ","
		end if
		fileHeader = fileHeader & treatmentList.item(treatmentId) & ","
	next

	' Control:
	' show response variable headers with result values
	for each responseId in responseList
		for each resultValue in resultList
			fileHeader = fileHeader & "c." & masterResultList.item(resultValue) & "." &  responseList.item(responseId) & ","
		next
	next

	' Non control:
	' show response variable headers with result values
	for each responseId in responseList
		for each resultValue in resultList
			fileHeader = fileHeader & masterResultList.item(resultValue) & "." &  responseList.item(responseId) & ","
		next
	next

	' add extra n to p ratio columns
	if calculateNtoPRatios then
		fileHeader = fileHeader & "shootNtoP,rootNtoP,totalTissueNtoP,availSoilNtoP,totalSoilNtoP,"
	end if		
	
	' add extra column headers for effect size columns
	if calcSumN or calcSumNRecip or calcLnResponseRatio then
		for each responseId in responseList
			if calcSumN then
				fileHeader = fileHeader & "sumN." & responseList.item(responseId) & ","
			end if
			if calcSumNRecip then
				fileHeader = fileHeader & "1/sumN." & responseList.item(responseId) &  ","
			end if
			if calcLnResponseRatio then
				fileHeader = fileHeader & "ln(rr)." & responseList.item(responseId) & ","
			end if
		next
	end if
	
	fileHeader = fileHeader & "!!end!!" & vbcrlf
	response.write fileHeader

	' loop through treatment sets
	do while not rsExpTrtSets.eof and (OUTPUT_ROW_LIMIT = 0 or outputRowCount <= OUTPUT_ROW_LIMIT)
		outputRowCount = outputRowCount + 1
		set row = new ResultRow
		
		' start capturing the treatment sets in this experiment
		' this will allow random selection of one set
		if lastExperimentId <> rsExpTrtSets("experimentId") and lastExperimentId <> -1 then
			
			' output
			' check if we're selecting a random treatment set
			if request.form("Export") = "Random Subset" then
				
				response.write "<h3>Experiment id #" & lastExperimentId & " Random Subset Selection:</h3>"
				
				' add control treatment set id's to 
				set expCtlTrtSetIds = Server.CreateObject("Scripting.Dictionary")
				for key = 0 to ubound(singleExperiment)
					treatmentSetId = singleExperiment(key).experimentDetails.item("ctlTrtSetId")
					if expCtlTrtSetIds.exists(CStr(treatmentSetId)) then
						expCtlTrtSetIds.item(CStr(treatmentSetId)) = expCtlTrtSetIds.item(CStr(treatmentSetId)) & "," & key
					else
						expCtlTrtSetIds.add CStr(treatmentSetId), key
					end if
				next
				
				' now pick out some keys
				for each treatmentSetId in expCtlTrtSetIds
					if OUTPUT_CSV_FILE = false then
						response.write "<h4>Control Treatment Set Id: " & treatmentSetId & "</h4>"
					end if
					
					tmpString = expCtlTrtSetIds.item(CStr(treatmentSetId))
					if instr(tmpString, ",") > 0 then
						' choose a random item here
						treatmentSetRows = split(tmpString, ",") 
						selectedRow = treatmentSetRows(Round(Rnd * ubound(treatmentSetRows)))
						if OUTPUT_CSV_FILE = false then
							response.write "<h5>Rows with same control: " & (ubound(treatmentSetRows) + 1) & " (" & tmpString & ")</h5>"
						end if
					else
						selectedRow = tmpString
					end if

					if OUTPUT_CSV_FILE = false then
						response.write "<h5>Selected row: " & selectedRow & "</h5>"
					end if
										
					' convert LSD to SD if necessary here
					if calcStdDev then
						ConvertLSDToSDForRow singleExperiment(selectedRow), expNumTreatmentLevels, expNumReplicates
					end if
					
					' calculate nitrogen to phosphorous ratios
					if calculateNtoPRatios then
						CalculateNitrogenToPhosphorous singleExperiment(selectedRow)
					end if
					
					' calculate values relating to effect size?
					if calcSumN or calcSumNRecip or calcLnResponseRatio then
						CalculateEffectSizeValues singleExperiment(selectedRow), calcSumN, calcSumNRecip, calcLnResponseRatio
					end if
					
					' now write it out
					OutputResultRow singleExperiment(selectedRow), responseList, resultList
					
				next

			' just output everything
			else
				For key=0 to ubound(singleExperiment)
					' convert LSD to SD if necessary here
					if calcStdDev then
						ConvertLSDToSDForRow singleExperiment(key), expNumTreatmentLevels, expNumReplicates
					end if

					' calculate nitrogen to phosphorous ratios
					if calculateNtoPRatios then
						CalculateNitrogenToPhosphorous singleExperiment(key)
					end if
					
					' calculate values relating to effect size?
					if calcSumN or calcSumNRecip or calcLnResponseRatio then
						CalculateEffectSizeValues singleExperiment(key), calcSumN, calcSumNRecip, calcLnResponseRatio
					end if

					OutputResultRow singleExperiment(key), responseList, resultList
				Next
			end if

			' reset stuff for LSD to SD conversion here
			if calcStdDev then			
				set expNumTreatmentLevels = nothing
				set expNumTreatmentLevels = Server.CreateObject("Scripting.Dictionary")
				' always add "control" for inoculum treatment by default
				expNumTreatmentLevels.add 4, "[control]"
				
				set expNumReplicates = nothing
				set expNumReplicates = Server.CreateObject("Scripting.Dictionary")				
			end if
			
			' reset array
			'response.write "--- <br />"
			redim singleExperiment(0)

			lastExperimentId = rsExpTrtSets("experimentId")
			'response.write "Experiment: " & rsExpTrtSets("experimentId") & " / " & rsExpTrtSets("lastName1") & " " & rsExpTrtSets("paperYear") & " " & rsExpTrtSets("experimentName") & "<br />"
			
		elseif lastExperimentId <> -1 then
			redim preserve singleExperiment(ubound(singleExperiment) + 1)
		else
			lastExperimentId = rsExpTrtSets("experimentId")
		end if
	
		' write each piece of data (field) from the treatment set properties
		for each field in rsExpTrtSets.fields
			'response.write "Field: " & field.name & "<br />"
			row.experimentDetails.add field.name, field.value
		next
				
		' get the control treatment details
		' only type of inoculum, because everything else will be equal
		' todo: remove this after verifying - it should always read "control"
		'row.experimentDetails = row.experimentDetails & "control,"
		row.experimentDetails.add "control", "control"
		'sqlCtlInoculum = "select [level] from expTrtSetSummary where treatmentId = 4 and id = " & rsExpTrtSets("ctlTrtSetId") & " order by displayOrder"
		'row.experimentDetails = row.experimentDetails & "<br><br>" & sqlCtlInoculum & "<br><br>"
		'rsTmp.Open sqlCtlInoculum, gConn, adOpenForwardOnly
		'if rsTmp.state = adStateOpen then
		'	row.experimentDetails = row.experimentDetails & rsTmp("level") & ","
		'	rsTmp.close
		'end if

		
		' convert units of soil P and soil N
		if convertSoilNAndPToPPM then
			if isnumeric(row.experimentDetails.item("soilNitrogen")) then
				value = row.experimentDetails.item("soilNitrogen")
				units = row.experimentDetails.item("soilNNutrientUnit")
				ConvertSoilPandNToPPM value, units
				row.experimentDetails.item("soilNitrogen") = value
				row.experimentDetails.item("soilNNutrientUnit") = units
			end if
			
			if isnumeric(row.experimentDetails.item("soilPhosphorous")) then
				value = row.experimentDetails.item("soilPhosphorous")
				units = row.experimentDetails.item("soilPNutrientUnit")
				ConvertSoilPandNToPPM value, units
				row.experimentDetails.item("soilPhosphorous") = value
				row.experimentDetails.item("soilPNutrientUnit") = units
			end if		
		end if

		
		''''''''''''''''''''''''''''''''''''''''''
		' get the treatment details
		''''''''''''''''''''''''''''''''''''''''''
		sqlNonCtlTreatments = "select [level], treatmentId, displayOrder from expTrtSetSummary where id = " & rsExpTrtSets("nonCtlTrtSetId") & " and treatmentId in (" & masterTreatmentIds & ") order by displayOrder"
		'response.write "Treatment sql: " & sqlNonCtlTreatments & "<br />"
		rsTreatmentLevels.Open sqlNonCtlTreatments, gConn, adOpenForwardOnly
		set tmpTreatList = Server.CreateObject("Scripting.Dictionary") 
		if rsTreatmentLevels.state = adStateOpen then
			' loop through all treatment levels
			do while not rsTreatmentLevels.EOF
				'wrap the level name to make sure we just get the exact level in the string searching in the next part
				levelString = "[" & CStr(rsTreatmentLevels("level")) & "]"
				'response.write "&nbsp;&nbsp;&nbsp;&nbsp;" & levelString & "<br />"
				treatmentId = rsTreatmentLevels("treatmentId")
				
				' check if the treatment is already in our experiment treatment list
				if expNumTreatmentLevels.exists(treatmentId) then
					
					allLevels = expNumTreatmentLevels.item(treatmentId)
					' check if this level has already been added
					if instr(allLevels, levelString) = 0 then
						allLevels = allLevels & "|" & levelString
						expNumTreatmentLevels.item(treatmentId) = allLevels 
					end if
				else
					expNumTreatmentLevels.add treatmentId, levelString
				end if
				
				'add this to our tmp list for use in the output control next
				tmpTreatList.add CStr(rsTreatmentLevels("treatmentId")), Trim(rsTreatmentLevels("level"))
								
				rsTreatmentLevels.movenext
			loop
			rsTreatmentLevels.close
		
		end if
		
		
		' loop through user selected treatments 
		' see if they exist in the temporary treatment list - if so, output them, if not, show a blank cell
		for each treatmentId in treatmentList
			if tmpTreatList.exists(treatmentId) then
				row.experimentDetails.add "treatmentId_" & treatmentId, tmpTreatList.item(treatmentId)
				'row.experimentDetails = row.experimentDetails & tmpTreatList.item(treatmentId)
			else
				row.experimentDetails.add "treatmentId_" & treatmentId, null
				'row.experimentDetails = row.experimentDetails & " "
			end if
			'row.experimentDetails = row.experimentDetails & ","				
		next
		
		
		''''''''''''''''''''''''''''''''''''''''''
		' results
		''''''''''''''''''''''''''''''''''''''''''

		' control set results
		sqlCtlResults = "select responseId, responseName, responseDisplayOrder "

		' only include the result values (mean, std dev etc..) that the user requested
		'for each resultValue in resultList
		'	sqlCtlResults = sqlCtlResults & ", [" & resultValue & "] "
		'next

		' include all values because we might have to calculate standard deviation later.
		for each resultValue in masterResultList.keys
			sqlCtlResults = sqlCtlResults & ", [" & resultValue & "] "
		next
		
		' this control set
		sqlCtlResults = sqlCtlResults & " from expResultSummary where expTrtSetId = " & rsExpTrtSets("ctlTrtSetId") & " and responseId in (" & requestedResponseIds & ") order by CStr(responseId)"
		'response.write "sql: " & sqlCtlResults & "<br />"
		
		rsCtlResults.Open sqlCtlResults, gConn, adOpenForwardOnly
		
		if rsCtlResults.state = adStateOpen then
			if not rsCtlResults.EOF then
				' first get the number of replicates
				if expNumReplicates.exists(rsExpTrtSets.fields("ctlTrtSetId").value) = false then
					expNumReplicates.add rsExpTrtSets.fields("ctlTrtSetId").value, Trim(rsCtlResults("numReplicates"))
					'response.write "Exp #" & rsExpTrtSets("experimentId") & " "
					'response.write "Control treatment #" & rsExpTrtSets.fields("ctlTrtSetId").value
					'response.write " results first row (n): "
					'response.write rsCtlResults("numReplicates")
					'response.write "<br />"
					'response.write "<strong>Added</strong><br />"					
				end if				
			end if
			
			'response.write "<br />Control results: " & sqlCtlResults & "<br />"
			
			' loop through responses - they should be in the same order (id)
			' see if they match this recordset, if not, output a blank cell (comma)
			' NOTE: if it seems like results are not showing up in the export, most
			' likely it is the sorting going on to match up dictionary object and recordset, then
			' the looping through each one to find the right items.
			' order here is CRITICAL! By assuming the sql and dictionary are in the same order,
			' it reduces the number of database calls, and therefore speeds things up.			
			for each responseId in responseList
				set tmpResult = new Result
				if not rsCtlResults.EOF then
					' match our result recordset respons to the user requested respons
					if responseId = CStr(rsCtlResults.fields("responseId").value) then
					
						tmpResult.mean = rsCtlResults("mean")
						tmpResult.stdError = rsCtlResults("stdError")						
						tmpResult.stdDev = rsCtlResults("stdDev")
						tmpResult.confInter95 = rsCtlResults("confInter95")
						tmpResult.lsd = rsCtlResults("lsd")
						tmpResult.numReplicates = rsCtlResults("numReplicates")
						
						' calculate standard deviation when required, and it does not already exist
						if calcStdDev = true and IsNull(tmpResult.stdDev) then
							if not IsNull(tmpResult.stdError) and not IsNull(tmpResult.numReplicates) then
								' Standard Deviation = Standard Error * Square Root (number of replicates)
								tmpResult.stdDev = tmpResult.stdError * Sqr(tmpResult.numReplicates)
							end if
						end if	
					
						' advance the recordset one record every time a user requested response id matches
						rsCtlResults.MoveNext					
					end if
				end if
				
				' add this result to the output as part of the array in the response key
				row.ctlResults.add responseId, tmpResult
			next
			rsCtlResults.close
		end if

		' non control results
		sqlNonCtlResults = "select responseId, responseName, responseDisplayOrder "

		' include all values because we might have to calculate standard deviation later.
		for each resultValue in masterResultList.keys
			sqlNonCtlResults = sqlNonCtlResults & ", [" & resultValue & "] "
		next
		
		' only results from non-control set
		sqlNonCtlResults = sqlNonCtlResults & " from expResultSummary where expTrtSetId = " & rsExpTrtSets("nonCtlTrtSetId") & " and responseId in (" & requestedResponseIds & ") order by CStr(responseId)"
		
		rsNonCtlResults.Open sqlNonCtlResults, gConn, adOpenForwardOnly
		
		if rsNonCtlResults.state = adStateOpen then

			if not rsNonCtlResults.EOF then
				' first get the number of replicates
				if not expNumReplicates.exists(rsExpTrtSets.fields("nonCtlTrtSetId").value) then
					expNumReplicates.add rsExpTrtSets.fields("nonCtlTrtSetId").value, Trim(rsNonCtlResults("numReplicates"))
					'response.write "Exp #" & rsExpTrtSets("experimentId") & " "
					'response.write "Inoculated treatment #" & rsExpTrtSets.fields("nonCtlTrtSetId").value
					'response.write " results first row (n): "
					'response.write rsNonCtlResults("numReplicates")
					'response.write "<br />"					
					'response.write "<strong>Added</strong><br />"
				end if				
			end if		
		
			' loop through responses - they should be in the same order (id)
			' as the dictionary object.
			' see if they match this recordset, if not, output a blank cell (comma)
			' NOTE: if it seems like results are not showing up in the export, most
			' likely it is the sorting going on to match up dictionary object and recordset, then
			' the looping through each one to find the right items.
			' order here is CRITICAL! By assuming the sql and dictionary are in the same order,
			' it reduces the number of database calls, and therefore speeds things up.

			'response.write "<br />Results: " & sqlNonCtlResults & "<br />"
			
			for each responseId in responseList
				set tmpResult = new Result
				
				'response.write "Looking for response responseId " & responseId & ": "
				if not rsNonCtlResults.EOF then				
				
					'response.write " rs responseId: " & CStr(rsNonCtlResults.fields("responseId").value)
					if responseId = CStr(rsNonCtlResults.fields("responseId").value) then

						tmpResult.mean = rsNonCtlResults("mean")
						tmpResult.stdError = rsNonCtlResults("stdError")						
						tmpResult.stdDev = rsNonCtlResults("stdDev")
						tmpResult.confInter95 = rsNonCtlResults("confInter95")
						tmpResult.lsd = rsNonCtlResults("lsd")
						tmpResult.numReplicates = rsNonCtlResults("numReplicates")
						
						' calculate standard deviation when required, and it does not already exist
						if calcStdDev = true and IsNull(tmpResult.stdDev) then
							if not IsNull(tmpResult.stdError) and not IsNull(tmpResult.numReplicates) then
								' Standard Deviation = Standard Error * Square Root (number of replicates)
								tmpResult.stdDev = tmpResult.stdError * Sqr(tmpResult.numReplicates)
							end if
						end if	
					
						' advance the recordset one record every time a user requested response id matches
						rsNonCtlResults.MoveNext
					end if
				end if
				
				' add this result to the output as part of the array in the response key
				row.results.add responseId, tmpResult
			next
			rsNonCtlResults.close
		end if
		
		' capture this row in the experiment array
		set singleExperiment(ubound(singleExperiment)) = row
		
		' go to next treatment
		rsExpTrtSets.movenext
	loop
	rsExpTrtSets.close
end if
set rsExpTrtSets = nothing


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' write a single row of comma separated values to client
'
' Parameters:
' myRow ResultRow single row results for exporting (control/non control treatment sets matched up)
' responses dictionary list of user requested responses, with responseId as key
' values dictionary list of use requested results to display (mean, standard deviation, etc...)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
sub OutputResultRow(byref myRow, byref responses, byref values)
	dim responseId, ctlOutput, nonCtlOutput, myTmp, expDetOutput, myVal, fieldName, calcColumnOutput
	
	if OUTPUT_DATA = false then exit sub
	
	for each fieldName in myRow.experimentDetails
		myVal = myRow.experimentDetails.item(fieldName)
		if len(myVal) > 0 then
			if isnumeric(myVal) then
				expDetOutput = expDetOutput & myVal
			else
				'strip commas from data
				myVal = replace(myVal, ",", ".")
				'strip returns
				myVal = replace(myVal, vbcrlf, " ")
				' add quotes
				expDetOutput = expDetOutput & myVal
			end if
		else
			' put in a space for no data - MetaWin can't handle a ",," in a CSV file - it needs ", ,"
			expDetOutput = expDetOutput & " "
		end if
		expDetOutput = expDetOutput & ","
	next
	
	' now loop through requested responses, and output the corresponding control results
	for each responseId in responses
		if myRow.ctlResults.exists(CStr(responseId)) then
			if values.exists("mean") then ctlOutput = ctlOutput &  myRow.ctlResults.item(responseId).mean & " ,"
			if values.exists("stdError") then ctlOutput = ctlOutput &  myRow.ctlResults.item(responseId).stdError & " ,"
			if values.exists("stdDev") then ctlOutput = ctlOutput &  myRow.ctlResults.item(responseId).stdDev & " ,"
			if values.exists("confInter95") then ctlOutput = ctlOutput &  myRow.ctlResults.item(responseId).confInter95 & " ,"
			if values.exists("lsd") then ctlOutput = ctlOutput &  myRow.ctlResults.item(responseId).lsd & " ,"
			if values.exists("numReplicates") then ctlOutput = ctlOutput &  myRow.ctlResults.item(responseId).numReplicates & " ,"
		else
			if values.exists("mean") then ctlOutput = ctlOutput &  " ,"
			if values.exists("stdError") then ctlOutput = ctlOutput &  " ,"
			if values.exists("stdDev") then ctlOutput = ctlOutput &  " ,"
			if values.exists("confInter95") then ctlOutput = ctlOutput &  " ,"
			if values.exists("lsd") then ctlOutput = ctlOutput &  " ,"
			if values.exists("numReplicates") then ctlOutput = ctlOutput &  " ,"	
		end if
		
		if myRow.results.exists(responseId) then
			if values.exists("mean") then nonCtlOutput = nonCtlOutput & myRow.results.item(responseId).mean & " ,"
			if values.exists("stdError") then nonCtlOutput = nonCtlOutput &  myRow.results.item(responseId).stdError & " ,"
			if values.exists("stdDev") then nonCtlOutput = nonCtlOutput &  myRow.results.item(responseId).stdDev & " ,"
			if values.exists("confInter95") then nonCtlOutput = nonCtlOutput &  myRow.results.item(responseId).confInter95 & " ,"
			if values.exists("lsd") then nonCtlOutput = nonCtlOutput &  myRow.results.item(responseId).lsd & " ,"
			if values.exists("numReplicates") then nonCtlOutput = nonCtlOutput &  myRow.results.item(responseId).numReplicates & " ,"
		else
			if values.exists("mean") then nonCtlOutput = nonCtlOutput &  " ,"
			if values.exists("stdError") then nonCtlOutput = nonCtlOutput &  " ,"
			if values.exists("stdDev") then nonCtlOutput = nonCtlOutput &  " ,"
			if values.exists("confInter95") then nonCtlOutput = nonCtlOutput &  " ,"
			if values.exists("lsd") then nonCtlOutput = nonCtlOutput &  " ,"
			if values.exists("numReplicates") then nonCtlOutput = nonCtlOutput &  " ,"	
		end if		
	next

	for each fieldName in myRow.calcColumns
		myVal = myRow.calcColumns.item(fieldName)
		if len(myVal) > 0 then
			if isnumeric(myVal) then
				calcColumnOutput = calcColumnOutput & myVal
			else
				'strip commas from data
				myVal = replace(myVal, ",", ".")
				'strip returns
				myVal = replace(myVal, vbcrlf, " ")
				' add quotes
				calcColumnOutput = calcColumnOutput & myVal
			end if
		else
			' put in a space for no data - MetaWin can't handle a ",," in a CSV file - it needs ", ,"
			calcColumnOutput = calcColumnOutput & " "
		end if
		calcColumnOutput = calcColumnOutput & ","
	next
	
	response.write expDetOutput & ctlOutput & nonCtlOutput & calcColumnOutput & "!!end!!" & vbcrlf
end sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' convert LSD to standard deviation for an experiment
' This requires the LSD numbers, the number of levels of each
' treatment in an experiment, and the number of replicates for each treatment set
'
' Parameters:
' row ResultRow contains the results for a single control/non control treatment set
' levels dictionary responseId keyed strings with | separated level names. This is for an entire experiment.
' replicates dictionary number of replicates keyed to treatment set id's (so that this number won't be double counted). This is also for an entire experiment
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
sub ConvertLSDToSDForRow(byref row, levels, replicates)
	dim trtId, setId, df, sumNRecip, sumN, x, z, lvl, tmpX, retval, valueCalced, output
	valueCalced = false
	
	' loop through experiment treatments
	x = 0
	z = 0
	for each trtId in levels
		'response.write "Treatment: " & trtId & " levels: " & levels.item(trtId) & "<br />"
		tmpX = ubound(split(levels.item(trtId), "|")) + 1
		if trtId = 4 then
			z = tmpX - 1
		else
			if x = 0 then
				x = tmpX
			else
				x = x * tmpX
			end if
		end if
	next
	
	if x > 0 then
		x = x - 1
	end if
		
	'reciprocals of n, summed
	sumNRecip = 0
	sumN = 0
	for each setId in replicates
		sumNRecip = sumNRecip + (1.0 / replicates.item(setId))
		sumN = sumN + replicates.item(setId)
	next
	
	df = sumN - z - x - (z * x)	
	
	' what to do when degrees of freedom is not greater than 1?
	if df > 1 then
		' turn df into an integer - so we can select from the T_TABLE
		df = round(df)
		' make sure df is no larger than highest T_TABLE array index
		if df > ubound(T_TABLE) then df = ubound(T_TABLE)
		
		' loop through LSD values
		for each trtId in row.ctlResults
			if isnumeric(row.ctlResults.item(trtId).lsd) then
				valueCalced = true
				row.ctlResults.item(trtId).stdDev = sqr( (row.ctlResults.item(trtId).lsd / T_TABLE(df) ) ^ 2 / (sumNRecip) )
				if OUTPUT_CSV_FILE = FALSE then
					output = output & "SD (ctl) = " & row.ctlResults.item(trtId).stdDev & " = sqrt( ( " & row.ctlResults.item(trtId).lsd & " / " & T_TABLE(df) & " ) ^ 2 / (" & sumNRecip & ") ) <br />"
				end if
			end if
		next

		for each trtId in row.results
			if isnumeric(row.results.item(trtId).lsd) then
				valueCalced = true
				row.results.item(trtId).stdDev = sqr( (row.results.item(trtId).lsd / T_TABLE(df) ) ^ 2 / (sumNRecip) )
				if OUTPUT_CSV_FILE = FALSE then
					output = output & "SD = " & row.results.item(trtId).stdDev & " = sqrt( ( " & row.results.item(trtId).lsd & " / " & T_TABLE(df) & " ) ^ 2 / (" & sumNRecip & ") ) <br />"
				end if
			end if
		next
		
	else
		if OUTPUT_CSV_FILE = FALSE then
			response.write "Experiment #" & row.experimentDetails.item("experimentId") & " df too small: " & df & "<br />"
		end if
	end if
	
	if OUTPUT_CSV_FILE = FALSE and valueCalced then
		response.write "<h2>Experiment #" & row.experimentDetails.item("experimentId") & "</h2>"
		response.write "<h3>"
		response.write " z = " & z & " / x = " & x & " / sum(n): " & sumN & " / sum(1/n): " & sumNRecip & " / df = " & df
		response.write "</h3>"
		response.write "<h5>" & output & "</h5>"
	end if
	
	'response.write "Experiment df: " & df & "<br />"
	
	'response.write "SD = sqrt( (LSD / t (" & df & ") ) ^ 2 / (" & sumNRecip & ") )<br />"
	
	'set ConvertLSDToSDForRow = row2
	
	'response.write "<br />"
end sub

function formatDate(format, intTimeStamp)
	Dim monthname()
	Redim monthname(12)
	monthname(1) = "January"
	monthname(2) = "February"
	monthname(3) = "March"
	monthname(4) = "April"
	monthname(5) = "May"
	monthname(6) = "June"
	monthname(7) = "July"
	monthname(8) = "August"
	monthname(9) = "September"
	monthname(10) = "October"
	monthname(11) = "November"
	monthname(12) = "December"
	
	dim unUDate, A
	
	dim OriginalLocale
	dim res
	OriginalLocale = GetLocale
	res = SetLocale("en-gb")
	
	' Test to see if intTimeStamp looks valid. If not, they have passed a normal date
	if not (isnumeric(intTimeStamp)) then
	if isdate(intTimeStamp) then
	intTimeStamp = DateDiff("S", "01/01/1970 00:00:00", intTimeStamp)
	else
	response.write "Date Invalid"
	exit function
	end if
	end if
	
	if (intTimeStamp=0) then
	unUDate = now()
	else
	unUDate = DateAdd("s", intTimeStamp, "01/01/1970 00:00:00")
	end if
	
	unUDate = trim(unUDate)
	
	'bug fix for midnight problems
	If (Len(unUDate) <= 11) Then unUDate = Trim(unUDate) & " 00:00:00"
	
	dim startM : startM = 1
	dim startD : startD = InStr(startM, unUDate, "/")+1
	dim startY : startY = InStr(startD, unUDate, "/")+1
	dim startHour : startHour = InStr(startY, unUDate, " ")+1
	dim startMin : startMin = InStr(startHour, unUDate, ":")+1
	dim startSec : startSec = InStr(startMin+1, unUDate, ":")+1
	dim dateMonth : dateMonth = mid(unUDate, startD, ((startY - 1) - startD))
	dim dateDay : dateDay = mid(unUDate, 1, ((startD - 1) - 1))
	dim dateYear : dateYear = Year(unUDate)
	dim dateHour : dateHour = mid(unUDate, startHour, ((startMin - startHour) - 1))
	dim dateMinute : dateMinute = mid(unUDate, startMin, 2)
	dim dateSecond : dateSecond = mid(unUDate, InStr(startMin, unUDate, ":") + 1, 2)
	
	format = replace(format, "%Y", right(dateYear, 4))
	format = replace(format, "%y", right(dateYear, 2))
	format = replace(format, "%m", dateMonth)
	format = replace(format, "%n", cint(dateMonth))
	
	' Response.Write CStr(cint(dateMonth))
	' Response.Flush
	
	format = replace(format, "%F", monthname(cint(dateMonth)))
	format = replace(format, "%M", left(monthname(cint(dateMonth)), 3))
	format = replace(format, "%d", dateDay)
	format = replace(format, "%j", cint(dateDay))
	format = replace(format, "%h", mid(unUDate, startHour, 2))
	format = replace(format, "%g", cint(mid(unUDate, startHour, 2)))
	
	if (cint(dateHour) > 12) then
	A = "PM"
	else
	A = "AM"
	end if
	format = replace(format, "%A", A)
	format = replace(format, "%a", lcase(A))
	
	if (A = "PM") then format = replace(format, "%H", Right("00" & dateHour - 12, 2))
	format = replace(format, "%H", dateHour)
	if (A = "PM") then format = replace(format, "%G", left("0" & cint(dateHour) - 12, 2))
	format = replace(format, "%G", cint(dateHour))
	
	format = replace(format, "%i", dateMinute)
	format = replace(format, "%I", cint(dateMinute))
	format = replace(format, "%s", dateSecond)
	format = replace(format, "%S", cint(dateSecond))
	format = replace(format, "%L", WeekDay(unUDate))
	format = replace(format, "%D", left(WeekDayName(WeekDay(unUDate)), 3))
	format = replace(format, "%l", WeekDayName(WeekDay(unUDate)))
	format = replace(format, "%U", intTimeStamp)
	format = replace(format, "11%O", "11th")
	format = replace(format, "1%O", "1st")
	format = replace(format, "12%O", "12th")
	format = replace(format, "2%O", "2nd")
	format = replace(format, "13%O", "13th")
	format = replace(format, "3%O", "3rd")
	format = replace(format, "%O", "th")
	
	formatDate = format
	res = SetLocale(OriginalLocale)

end function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' convert soil phosphorous or soil nitrogen to
' parts per million
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''
sub ConvertSoilPandNToPPM(byref value, byref unit)
	dim convFactor 'the conversion factor for converting units of soil P and N

	if not OUTPUT_CSV_FILE then
		response.write "ConvertSoilPandNToPPM: " & value & " " & unit & " -&gt; "
	end if
	select case unit
		case "ppm"
			convFactor = 1
		case "ug/g"
			convFactor = 1
		case "mg/g"
			convFactor = 1 / 1000
		case "mg/kg"
			convFactor = 1
		case "mg/L"
			convFactor = 1 'is this right?
		case "kg/ha"
			convFactor = 1 / 2.24
		case "%"
			convFactor = 10000
		case "g/kg"
			convFactor = 1000
		case else
			convFactor = 1
	end select
	value = value * convFactor
	unit = "ppm"
	if not OUTPUT_CSV_FILE then
		response.write "&nbsp; " & value & " " & unit & "<br />"
	end if
end sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' calculate requested effect size values
' for a given row
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
sub CalculateEffectSizeValues(byref row, doSumN, doSumNRecip, doLnResponseRatio)
	dim responseId, sum, recip, rr
	
	for each responseId in row.ctlResults
		if not isnull(row.ctlResults.item(responseId).numReplicates) and not isnull(row.results.item(responseId).numReplicates) then
			if doSumN then
				row.calcColumns.add "sumN_" & responseId,  row.ctlResults.item(responseId).numReplicates + row.results.item(responseId).numReplicates
			end if
			
			if doSumNRecip then
				row.calcColumns.add "sumNRecip_" & responseId,  1 / (row.ctlResults.item(responseId).numReplicates + row.results.item(responseId).numReplicates)
			end if
		else
			if doSumN then
				row.calcColumns.add "sumN_" & responseId, null
			end if
			
			if doSumNRecip then
				row.calcColumns.add "sumNRecip_" & responseId, null
			end if		
		end if
			
		if not isnull(row.ctlResults.item(responseId).mean) and not isnull(row.results.item(responseId).mean) then		
			if doLnResponseRatio then
				if row.ctlResults.item(responseId).mean > 0 then
					row.calcColumns.add "lnResponseRatio_" & responseId, log(row.results.item(responseId).mean / row.ctlResults.item(responseId).mean)
				else
					row.calcColumns.add "lnResponseRatio_" & responseId, null
				end if
			end if
		else
			if doLnResponseRatio then
				row.calcColumns.add "lnResponseRatio_" & responseId, null
			end if
		end if
	next

end sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' calculate nitrogen to phosphorous ratios
' for a given row
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
sub CalculateNitrogenToPhosphorous(byref row)
	dim shootNtoP, rootNtoP, totalTissueNtoP, availSoilNtoP, totalSoilNtoP
	
	' shoot n to p
	' contents
	if not isnull(row.results.item("24").mean) and not isnull(row.results.item("26").mean) then
		shootNtoP = row.results.item("24").mean / row.results.item("26").mean
	' concentrations
	elseif not isnull(row.results.item("21").mean) and not isnull(row.results.item("29").mean) then
		shootNtoP = row.results.item("21").mean / row.results.item("29").mean
	end if
	
	' root n to p
	' contents
	if not isnull(row.results.item("41").mean) and not isnull(row.results.item("27").mean) then
		rootNtoP = row.results.item("41").mean / row.results.item("27").mean
	' concentrations
	elseif not isnull(row.results.item("20").mean) and not isnull(row.results.item("28").mean) then
		rootNtoP = row.results.item("20").mean / row.results.item("28").mean
	end if
	
	' total tissue n to p
	'whole plant concentrations
	if not isnull(row.results.item("14").mean) and not isnull(row.results.item("15").mean) then
		totalTissueNtoP = row.results.item("14").mean / row.results.item("15").mean
	'whole plant contents
	elseif not isnull(row.results.item("37").mean) and not isnull(row.results.item("23").mean) then
		totalTissueNtoP = row.results.item("37").mean / row.results.item("23").mean
	'root and shoot contents
	elseif not isnull(row.results.item("24").mean) and not isnull(row.results.item("26").mean) and not isnull(row.results.item("41").mean) and not isnull(row.results.item("27").mean) then
		
		totalTissueNtoP = (row.results.item("24").mean + row.results.item("41").mean) / (row.results.item("26").mean + row.results.item("27").mean)

	'root and shoot concentrations
	elseif not isnull(row.results.item("21").mean) and not isnull(row.results.item("29").mean) and not isnull(row.results.item("20").mean) and not isnull(row.results.item("28").mean) then
	
		totalTissueNtoP = (row.results.item("21").mean + row.results.item("20").mean) / (row.results.item("29").mean + row.results.item("28").mean)
	end if
	
	' total and avail soil N to P
	' first check if the n and p in the soil are defined
	' and the type is defined and the types are equal
	if not isnull(row.experimentDetails.item("soilNitrogen")) and not isnull(row.experimentDetails.item("soilPhosphorous")) and not isnull(row.experimentDetails.item("soilNNutrientType")) and not isnull(row.experimentDetails.item("soilPNutrientType")) and row.experimentDetails.item("soilNNutrientType") = row.experimentDetails.item("soilPNutrientType") then
		' now check for "total" soil p and n
		if row.experimentDetails.item("soilPNutrientType") = "Total" and row.experimentDetails.item("soilPhosphorous") > 0 then
			
			totalSoilNtoP = row.experimentDetails.item("soilNitrogen") / row.experimentDetails.item("soilPhosphorous")
			'response.write "<h3>Total: " & totalSoilNtoP & "</h3>"
		elseif row.experimentDetails.item("soilPNutrientType") = "Available/Extractable/Mobile" and row.experimentDetails.item("soilPhosphorous") > 0 then
			
			availSoilNtoP = row.experimentDetails.item("soilNitrogen") / row.experimentDetails.item("soilPhosphorous")
			'response.write "<h3>Avail: " & availSoilNtoP & "</h3>"
		end if
		
	end if
	
	' add columns to as experiment details items
	row.calcColumns.add "shootNtoP", shootNtoP
	row.calcColumns.add "rootNtoP", rootNtoP
	row.calcColumns.add "totalTissueNtoP", totalTissueNtoP
	row.calcColumns.add "availSoilNtoP", availSoilNtoP
	row.calcColumns.add "totalSoilNtoP", totalSoilNtoP
	
end sub
%>
